'use strict';

module.exports = function (babel) {
    var t = babel.types;

    return {
        visitor: {
            CallExpression: function CallExpression(path) {
                if (calleeDoesMatch(t, path.node.callee) && argumentDoesMatch(t, path.node.arguments)) {

                    replacePath(t, path);
                }
            }
        }
    };
};

function calleeDoesMatch(t, callee) {
    return t.isMemberExpression(callee) && callee.object && callee.property && t.isIdentifier(callee.object, { name: 'React' }) && t.isIdentifier(callee.property, { name: 'createElement' });
}

function argumentDoesMatch(t, nodeArguments) {
    var config = (nodeArguments || [])[1];

    if (t.isCallExpression(config)) {
        // <Component propA={...} propB={...} {...otherProps} />
        var configArguments = config.arguments;
        for (var i = 0; i < configArguments.length; i++) {
            var argument = configArguments[i];
            // <Component propA={...} propB={...} />
            if (t.isObjectExpression(argument)) {
                if (objectHasPropertyMatch(t, argument)) {
                    return true;
                }
            }
            if (t.isIdentifier(argument)) {
                // <Component {...otherProps} />
                // check scope to resolve otherProps key/value pairs
                // const variableName = argument.name;
            }
        }
    }

    if (t.isObjectExpression(config)) {
        // React.createElement(type, config, children)
        return objectHasPropertyMatch(t, config);
    }
}

function objectHasPropertyMatch(t, object) {
    var properties = object.properties || [];
    return !!properties.filter(function (p) {
        return propertyDoesMatch(t, p);
    }).length;
}

function propertyDoesMatch(t, property) {
    return t.isObjectProperty(property) && t.isIdentifier(property.key, { name: 'cache' }) && cachePropertyDoesMatch(t, property.value);
}

function cachePropertyDoesMatch(t, value) {
    console.log(value);
    return t.isBooleanLiteral(value, { value: true }) || t.isStringLiteral(value) || t.isFunction(value);
}

var key = -1;
function next() {
    return ++key;
}

function replacePath(t, path) {
    path.replaceWith(t.callExpression(t.callExpression(t.memberExpression(t.identifier('ReactCache'), t.identifier('createElement')), [t.numericLiteral(next())]), path.node.arguments));
}